/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.mojo.translate.omegat;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.shared.model.fileset.FileSet;
import org.codehaus.plexus.util.FileUtils;

/**
 * Base class for bulk copying of sources and translations.
 * 
 * @author johan
 */
public abstract class AbstractTransferMojo extends AbstractMojo {
    
    /**
     * The Maven Project.
     *
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    protected MavenProject project;

    /**
     * The Maven Session Object.
     *
     * @parameter expression="${session}"
     * @required
     * @readonly
     */
    protected MavenSession session;
    
    /**
     * Location of the translation projects root directory.
     * 
     * The interpretation of the path depends on the <code>translationRootType</code>.
     * 
     * @parameter default-value="translations/omegat"
     * @required
     */
    private String translationRootPath;
    
    /**
     * Indicates how the <code>translationRootPath</code> will be interpreted.
     * 
     * Possible values:
     * <ul>
     * <li><code>absolute</code></li>
     * <li><code>project<c/ode>: relative to the module being processed; will create translations per module</li>
     * <li><code>session</code>: relative to the project from which the maven session was started (execution root);
     *      this will aggregate translations under this (parent) project.</li>
     * </ul>
     * 
     * @parameter default-value="session"
     * @required
     */
    private String translationRootType;

    /**
     * A list of <code>fileSet</code> rules to select files and directories.
     *
     * Sources will be fetched from the fileset's <code>directory</code>.
     * Translations will be copied to the fileset's <code>outputDirectory</code>.
     * 
     * @parameter
     * @required
     */
//    protected List<FileSet> filesets;
    // http://jira.codehaus.org/browse/MSHARED-96
    protected FileSet[] filesets;
    
    /**
     * Source language, the language used by the non-suffixed files.
     * 
     * A locale string.
     *
     * @parameter default-value="en" alias="sourceLocale"
     */
    private String sourceLanguage;
    
    /**
     * Target languages, the language(s) to which the files are to be translated.
     * 
     * A comma separated list of locales.
     * 
     * @parameter default-value="nl" alias="targetLocales"
     */
    private String targetLanguages;
    
    // validated parameters:
    private File translationRootDirectory;
    private String sourceLocale;
    private List<String> targetLocales = new LinkedList<>();
    
    
    protected enum TranslationRootType { 
        SESSION, // relative to execution root
        PROJECT, // relative to current project/module
        ABSOLUTE 
    }

    /**
     * Validate input and convert to useful types.
     * 
     * @throws MojoExecutionException 
     */
    protected void init() throws MojoExecutionException {
        validateLocales();
        validateFilesets();
        setTranslationRootDirectory();
    }
    
    // validate (and set) locales from input language parameters:
    private void validateLocales() throws MojoExecutionException {
        sourceLocale = Utils.validateLocaleString(sourceLanguage);
        String[] locales = targetLanguages.split(",");
        for (String locale : locales) {
            targetLocales.add(Utils.validateLocaleString(locale));
            if (locale.equals(sourceLanguage)) {
                throw new MojoExecutionException("Target " + locale + " equals source.");
            }
        }
    }
    
    // check required nested elements:
    private void validateFilesets() throws MojoExecutionException {
        if  (filesets == null || filesets.length == 0) {
            throw new MojoExecutionException("Fileset configuration elements are missing or empty.");
        }
        for (FileSet fileset : filesets) {
            String in = fileset.getDirectory(), out = fileset.getOutputDirectory();
            if (in == null || in.isEmpty()) {
                throw new MojoExecutionException("Fileset (input) directory parameter is missing or empty.");
            }
            if (out == null || out.isEmpty()) {
                throw new MojoExecutionException("Fileset outputDirectory parameter is missing or empty.");
            }
        }
    }    

    protected File getTranslationRootDirectory() {
        return translationRootDirectory;
    }

    protected String getSourceLocale() {
        return sourceLocale;
    }

    protected List<String> getTargetLocales() {
        return targetLocales;
    }
    
    private void setTranslationRootDirectory() throws MojoExecutionException {
        getLog().debug("Determining translation root directory.");
        getLog().debug("Type: "+ translationRootType);
        getLog().debug("Path: "+ translationRootPath);

        TranslationRootType type;
        try {
            type = TranslationRootType.valueOf(translationRootType.toUpperCase());
        } catch (NullPointerException | IllegalArgumentException e) {
            throw new MojoExecutionException("Invalid translation root type: " + translationRootType);
        }
        
        File baseDir;
        
        switch(type) {
            case SESSION:
                baseDir = new File(session.getExecutionRootDirectory());
                break;
            case PROJECT:
                baseDir = project.getBasedir();
                break;
            case ABSOLUTE:
            default:
                baseDir = null;
        }
        getLog().debug(baseDir.getAbsolutePath());
        translationRootDirectory = new File(baseDir, translationRootPath);
        getLog().debug(translationRootDirectory.getAbsolutePath());
    }
    
    protected void copyFile(File from, File to) throws MojoExecutionException {
        try {
            FileUtils.copyFile(from, to);
        } catch (IOException ex) {
            getLog().debug("from: " + from.getAbsolutePath());
            getLog().debug("to: " + to.getAbsolutePath());
            throw new MojoExecutionException("Error copying file", ex);
        }
    }
}
